import 'package:flutter/material.dart';
import 'package:im_flutter_sdk/im_flutter_sdk.dart';

///
///
///
/// 服务器连接监听类。
///
/// 对于不稳定网络条件下的onDisconnected情况，您不需要手动重新连接，
/// SDK会自动处理。
///
/// 添加 connection event handler:
/// ```dart
///   EMClient.getInstance.addConnectionEventHandler(UNIQUE_HANDLER_ID, EMConnectionEventHandler());
/// ```
///
/// 移除 connection event handler:
/// ```dart
///   EMClient.getInstance.removeConnectionEventHandler(UNIQUE_HANDLER_ID);
/// ```
///
class EMConnectionEventHandler {
  ///
  ///
  ///
  /// 成功连接到 chat 服务器时触发的回调。
  ///
  final VoidCallback? onConnected;

  ///
  ///
  ///
  /// 与 chat 服务器断开连接时触发的回调。
  ///
  final VoidCallback? onDisconnected;

  ///
  ///
  ///
  /// 其他设备登录回调。
  ///
  final void Function(String deviceName)? onUserDidLoginFromOtherDevice;

  ///
  ///
  ///
  /// 当前用户被服务器移除回调。
  ///
  final VoidCallback? onUserDidRemoveFromServer;

  ///
  ///
  ///
  /// 被服务器禁止连接回调。
  ///
  final VoidCallback? onUserDidForbidByServer;

  ///
  ///
  ///
  /// 用户密码变更回调。
  ///
  final VoidCallback? onUserDidChangePassword;

  ///
  ///
  ///
  /// 登录设备过多回调。
  ///
  final VoidCallback? onUserDidLoginTooManyDevice;

  ///
  ///
  ///
  /// 被其他设备踢掉回调。
  ///
  final VoidCallback? onUserKickedByOtherDevice;

  ///
  ///
  ///
  /// 鉴权失败回调。
  ///
  final VoidCallback? onUserAuthenticationFailed;

  ///
  ///
  ///
  /// Agora token 即将过期时触发。
  ///
  final VoidCallback? onTokenWillExpire;

  ///
  ///
  ///
  /// Agora token 已过期时触发。
  ///
  final VoidCallback? onTokenDidExpire;

  ///
  ///
  ///
  /// 应用程序的日活跃用户数量（DAU）或月活跃用户数量（MAU）达到上限。
  ///
  final VoidCallback? onAppActiveNumberReachLimit;

  ///
  ///
  ///
  /// 连接状态监听。
  ///
  /// Param [onConnected] 成功连接到 chat 服务器时触发的回调。
  ///
  /// Param [onDisconnected] 和 chat 服务器断开连接时触发的回调。
  ///
  /// Param [onUserDidLoginFromOtherDevice] 其他设备登录回调。
  ///
  /// Param [onUserDidRemoveFromServer] 被服务器移除回调。
  ///
  /// Param [onUserDidForbidByServer] 被服务器禁止连接回调。
  ///
  /// Param [onUserDidChangePassword] 用户密码变更回调。
  ///
  /// Param [onUserDidLoginTooManyDevice] 登录设备过多回调。
  ///
  /// Param [onUserKickedByOtherDevice] 被其他设备踢掉回调。
  ///
  /// Param [onUserAuthenticationFailed] 鉴权失败回调。
  ///
  /// Param [onTokenWillExpire] Agora token 即将过期时回调。
  ///
  /// Param [onTokenDidExpire] Agora token 已过期时回调。
  ///
  /// Param [onAppActiveNumberReachLimit] 应用程序的日活跃用户数量（DAU）或月活跃用户数量（MAU）达到上限时回调。
  ///
  EMConnectionEventHandler({
    this.onConnected,
    this.onDisconnected,
    this.onUserDidLoginFromOtherDevice,
    this.onUserDidRemoveFromServer,
    this.onUserDidForbidByServer,
    this.onUserDidChangePassword,
    this.onUserDidLoginTooManyDevice,
    this.onUserKickedByOtherDevice,
    this.onUserAuthenticationFailed,
    this.onTokenWillExpire,
    this.onTokenDidExpire,
    this.onAppActiveNumberReachLimit,
  });
}

///
///
///
/// 多设备事件监听
/// 监听当前用户在其他设备上的操作的回调，包括联系人更改、群组和 thread 等更改。
///
/// 添加监听:
/// ```dart
///   EMClient.getInstance.addMultiDeviceEventHandler((UNIQUE_HANDLER_ID, EMMultiDeviceEventHandler());
/// ```
///
/// 移除监听:
/// ```dart
///   EMClient.getInstance.removeMultiDeviceEventHandler(UNIQUE_HANDLER_ID);
/// ```
///
class EMMultiDeviceEventHandler {
  ///
  ///
  ///
  /// 多设备联系人事件。
  ///
  final void Function(
    EMMultiDevicesEvent event,
    String userId,
    String? ext,
  )? onContactEvent;

  ///
  ///
  ///
  /// 多设备群组事件。
  ///
  final void Function(
    EMMultiDevicesEvent event,
    String groupId,
    List<String>? userIds,
  )? onGroupEvent;

  ///
  ///
  ///
  /// 多设备 Thread 事件。
  ///
  final void Function(
    EMMultiDevicesEvent event,
    String chatThreadId,
    List<String> userIds,
  )? onChatThreadEvent;

  ///
  ///
  ///
  /// 开启多设备后对单个会话删除漫游消息后对其他设备的回调。
  ///
  final void Function(
    String conversationId,
    String deviceId,
  )? onRemoteMessagesRemoved;

  ///
  ///
  ///
  /// 开启多设备后单个会话操作的多设备事件回调。
  ///
  final void Function(
    EMMultiDevicesEvent event,
    String conversationId,
    EMConversationType type,
  )? onConversationEvent;

  ///
  ///
  ///
  /// 多设备事件。
  ///
  /// Param [onContactEvent] 多设备联系人事件。
  ///
  /// Param [onGroupEvent] 多设备群组事件。
  ///
  /// Param [onChatThreadEvent] 多设备 Thread 事件。
  ///
  /// Param [onRemoteMessagesRemoved] 多设备漫游消息删除事件。
  ///
  /// Param [onConversationEvent] 多设备单个会话操作事件。
  ///
  ///
  EMMultiDeviceEventHandler({
    this.onContactEvent,
    this.onGroupEvent,
    this.onChatThreadEvent,
    this.onRemoteMessagesRemoved,
    this.onConversationEvent,
  });
}

///
///
///
/// Chat 事件监听。
///
/// 用于监听收消息，已读回执，等回调。
///
/// 添加监听：
/// ```dart
///   EMClient.getInstance.chatManager.addEventHandler(UNIQUE_HANDLER_ID, EMChatEventHandler());
/// ```
///
/// 移除监听：
/// ```dart
///   EMClient.getInstance.chatManager.removeEventHandler(UNIQUE_HANDLER_ID);
/// ```
///
class EMChatEventHandler {
  ///
  ///
  ///
  /// 收到消息回调。
  /// 在收到文本、图片、视频、语音、地理位置和文件等消息时，通过此回调通知用户。
  ///
  final void Function(List<EMMessage> messages)? onMessagesReceived;

  ///
  ///
  ///
  /// 收到命令消息回调。
  /// 与 [onMessagesReceived] 不同, 这个回调只包含命令的消息，命令消息通常不对用户展示。
  ///
  final void Function(List<EMMessage> messages)? onCmdMessagesReceived;

  ///
  ///
  ///
  /// 收到单聊消息已读回执的回调。
  ///
  final void Function(List<EMMessage> messages)? onMessagesRead;

  ///
  ///
  ///
  /// 收到群组消息的已读回执的回调。
  ///
  final void Function(List<EMGroupMessageAck> groupMessageAcks)?
      onGroupMessageRead;

  ///
  ///
  ///
  /// 群消息已读变更。
  ///
  final VoidCallback? onReadAckForGroupMessageUpdated;

  ///
  ///
  ///
  /// 收到消息已送达回执的回调。
  ///
  final void Function(List<EMMessage> messages)? onMessagesDelivered;

  ///
  ///
  ///
  /// 已收到的消息被撤回的回调。
  ///
  final void Function(List<EMMessage> messages)? onMessagesRecalled;

  ///
  ///
  ///
  /// 会话更新事件回调。
  ///
  final VoidCallback? onConversationsUpdate;

  ///
  ///
  ///
  /// 收到会话已读回执的回调。
  ///
  /// 回调此方法的场景：
  /// （1）消息被接收方阅读，即接收方发送了会话已读回执。
  /// SDK 在接收到此事件时，会将本地数据库中该会话中消息的 `isAcked` 属性置为 `true`。
  /// （2）多端多设备登录场景下，一端发送会话已读回执，服务器端会将会话的未读消息数置为 0，
  /// 同时其他端会回调此方法，并将本地数据库中该会话中消息的 `isRead` 属性置为 `true`。
  ///
  final void Function(String from, String to)? onConversationRead;

  ///
  ///
  ///
  /// 消息表情回复（Reaction）变化监听器。
  ///
  final void Function(List<EMMessageReactionEvent> events)?
      onMessageReactionDidChange;

  ///
  ///
  ///
  /// 收到消息内容变化。
  ///
  final void Function(
    EMMessage message,
    String operatorId,
    int operationTime,
  )? onMessageContentChanged;

  ///
  ///
  ///
  /// 消息事件监听。
  ///
  /// Param [onMessagesReceived] 在收到文本、图片、视频、语音、地理位置和文件等消息时，通过此回调通知用户。
  ///
  /// Param [onCmdMessagesReceived] 收到命令消息回调。
  ///
  /// Param [onMessagesRead] 收到单聊消息已读回执的回调。
  ///
  /// Param [onGroupMessageRead] 收到群组消息的已读回执的回调。
  ///
  /// Param [onReadAckForGroupMessageUpdated] 群消息已读变更。
  ///
  /// Param [onMessagesDelivered] 收到消息已送达回执的回调。
  ///
  /// Param [onMessagesRecalled] 已收到的消息被撤回的回调。
  ///
  /// Param [onConversationsUpdate] 会话更新事件回调。
  ///
  /// Param [onConversationRead] 收到会话已读回执的回调。
  ///
  /// Param [onMessageReactionDidChange] 消息表情回复（Reaction）变化监听器。
  ///
  /// Param [onMessageContentChanged] 收到消息内容变化。
  ///
  EMChatEventHandler({
    this.onMessagesReceived,
    this.onCmdMessagesReceived,
    this.onMessagesRead,
    this.onGroupMessageRead,
    this.onReadAckForGroupMessageUpdated,
    this.onMessagesDelivered,
    this.onMessagesRecalled,
    this.onConversationsUpdate,
    this.onConversationRead,
    this.onMessageReactionDidChange,
    this.onMessageContentChanged,
  });
}

///
///
///
/// 聊天室事件监听。
///
/// 添加监听:
/// ```dart
///   EMClient.getInstance.chatRoomManager.addEventHandler(UNIQUE_HANDLER_ID, EMChatRoomEventHandler());
/// ```
///
/// Removes a chat room event handler:
/// ```dart
///   EMClient.getInstance.chatRoomManager.removeEventHandler(UNIQUE_HANDLER_ID);
/// ```
///
class EMChatRoomEventHandler {
  ///
  ///
  ///
  /// 有成员设置为聊天室管理员的回调。
  ///
  final void Function(
    String roomId,
    String admin,
  )? onAdminAddedFromChatRoom;

  ///
  ///
  ///
  /// 移除聊天室管理员权限的回调。
  ///
  final void Function(
    String roomId,
    String admin,
  )? onAdminRemovedFromChatRoom;

  ///
  ///
  ///
  /// 聊天室全员禁言状态变化回调。
  ///
  final void Function(
    String roomId,
    bool isAllMuted,
  )? onAllChatRoomMemberMuteStateChanged;

  ///
  ///
  ///
  /// 有成员被加入聊天室白名单的回调。
  ///
  final void Function(
    String roomId,
    List<String> members,
  )? onAllowListAddedFromChatRoom;

  ///
  ///
  ///
  /// 有成员被移出聊天室白名单的回调。
  ///
  final void Function(
    String roomId,
    List<String> members,
  )? onAllowListRemovedFromChatRoom;

  ///
  ///
  ///
  /// 聊天室公告更新回调。
  ///
  final void Function(
    String roomId,
    String announcement,
  )? onAnnouncementChangedFromChatRoom;

  ///
  ///
  ///
  /// 聊天室解散的回调。
  ///
  final void Function(
    String roomId,
    String? roomName,
  )? onChatRoomDestroyed;

  ///
  ///
  ///
  /// 聊天室成员主动退出回调。
  ///
  final void Function(
    String roomId,
    String? roomName,
    String participant,
  )? onMemberExitedFromChatRoom;

  ///
  ///
  ///
  /// 聊天室加入新成员回调。
  ///
  final void Function(String roomId, String participant)?
      onMemberJoinedFromChatRoom;

  ///
  ///
  ///
  /// 有成员被禁言回调。
  ///
  final void Function(
    String roomId,
    List<String> mutes,
    String? expireTime,
  )? onMuteListAddedFromChatRoom;

  ///
  ///
  ///
  /// 有成员从禁言列表中移除回调。
  ///
  final void Function(
    String roomId,
    List<String> mutes,
  )? onMuteListRemovedFromChatRoom;

  ///
  ///
  ///
  /// 转移聊天室的所有权的回调。
  ///
  final void Function(
    String roomId,
    String newOwner,
    String oldOwner,
  )? onOwnerChangedFromChatRoom;

  ///
  ///
  ///
  /// 当前用户被移出聊天室回调。
  ///
  final void Function(
    String roomId,
    String? roomName,
    String? participant,
    LeaveReason? reason,
  )? onRemovedFromChatRoom;

  ///
  ///
  ///
  /// 聊天室详情变更。
  ///
  final void Function(EMChatRoom room)? onSpecificationChanged;

  ///
  ///
  ///
  /// 聊天室属性变更。
  ///
  final void Function(
    String roomId,
    Map<String, String> attributes,
    String from,
  )? onAttributesUpdated;

  ///
  ///
  ///
  /// 聊天室属性被删除。
  ///
  final void Function(
    String roomId,
    List<String> removedKeys,
    String from,
  )? onAttributesRemoved;

  ///
  ///
  ///
  /// 聊天室事件监听。
  ///
  /// Param [onAdminAddedFromChatRoom] 有成员设置为聊天室管理员的回调。
  ///
  /// Param [onAdminRemovedFromChatRoom] 移除聊天室管理员权限的回调。
  ///
  /// Param [onAllChatRoomMemberMuteStateChanged] 聊天室全员禁言状态变化回调。
  ///
  /// Param [onAllowListAddedFromChatRoom] 有成员被加入聊天室白名单的回调。
  ///
  /// Param [onAllowListRemovedFromChatRoom] 有成员被移出聊天室白名单的回调。
  ///
  /// Param [onAnnouncementChangedFromChatRoom] 聊天室公告更新回调。
  ///
  /// Param [onChatRoomDestroyed] 聊天室解散的回调。
  ///
  /// Param [onMemberExitedFromChatRoom] 聊天室成员主动退出回调。
  ///
  /// Param [onMemberJoinedFromChatRoom] 聊天室加入新成员回调。
  ///
  /// Param [onMuteListAddedFromChatRoom] 有成员被禁言回调。
  ///
  /// Param [onMuteListRemovedFromChatRoom] 有成员从禁言列表中移除回调。
  ///
  /// Param [onOwnerChangedFromChatRoom] 转移聊天室的所有权的回调。
  ///
  /// Param [onRemovedFromChatRoom] 聊天室成员被移出聊天室回调。
  ///
  /// Param [onSpecificationChanged] 聊天室详情变更。
  ///
  /// Param [onAttributesUpdated] 聊天室属性变更。
  ///
  /// Param [onAttributesRemoved] 聊天室属性被删除。
  ///
  EMChatRoomEventHandler({
    this.onAdminAddedFromChatRoom,
    this.onAdminRemovedFromChatRoom,
    this.onAllChatRoomMemberMuteStateChanged,
    this.onAllowListAddedFromChatRoom,
    this.onAllowListRemovedFromChatRoom,
    this.onAnnouncementChangedFromChatRoom,
    this.onChatRoomDestroyed,
    this.onMemberExitedFromChatRoom,
    this.onMemberJoinedFromChatRoom,
    this.onMuteListAddedFromChatRoom,
    this.onMuteListRemovedFromChatRoom,
    this.onOwnerChangedFromChatRoom,
    this.onRemovedFromChatRoom,
    this.onSpecificationChanged,
    this.onAttributesUpdated,
    this.onAttributesRemoved,
  });
}

///
///
///
/// Thread 事件监听
///
/// 添加监听:
/// ```dart
///   EMClient.getInstance.chatThreadManager.addEventHandler(UNIQUE_HANDLER_ID, EMChatThreadEventHandler());
/// ```
///
/// 移除监听:
/// ```dart
/// EMClient.getInstance.chatThreadManager.removeEventHandler(UNIQUE_HANDLER_ID);
/// ```
///
class EMChatThreadEventHandler {
  ///
  ///
  ///
  /// 子区创建回调。
  ///
  final void Function(
    EMChatThreadEvent event,
  )? onChatThreadCreate;

  ///
  ///
  ///
  /// 子区解散事件。
  /// 子区所属群组的所有成员均可调用该方法。
  ///
  final void Function(
    EMChatThreadEvent event,
  )? onChatThreadDestroy;

  ///
  ///
  ///
  /// 子区更新回调。
  /// 子区所属群组的所有成员均可调用该方法。
  ///
  final void Function(
    EMChatThreadEvent event,
  )? onChatThreadUpdate;

  ///
  ///
  ///
  /// 管理员移除子区用户的回调。
  ///
  final void Function(
    EMChatThreadEvent event,
  )? onUserKickOutOfChatThread;

  ///
  ///
  ///
  /// Thread 事件监听。
  ///
  /// Param [onChatThreadCreate] 子区创建回调。
  ///
  /// Param [onChatThreadDestroy] 子区解散事件, 子区所属群组的所有成员均可调用该方法。
  ///
  /// Param [onChatThreadUpdate] 子区更新回调, 子区所属群组的所有成员均可调用该方法。
  ///
  /// Param [onUserKickOutOfChatThread] 管理员移除子区用户的回调。
  ///
  EMChatThreadEventHandler({
    this.onChatThreadCreate,
    this.onChatThreadDestroy,
    this.onChatThreadUpdate,
    this.onUserKickOutOfChatThread,
  });
}

///
///
///
/// 联系人事件监听
///
/// 添加监听:
/// ```dart
///   EMClient.getInstance.contactManager.addEventHandler(UNIQUE_HANDLER_ID, EMContactEventHandler());
/// ```
///
/// 移除监听:
/// ```dart
///   EMClient.getInstance.contactManager.removeEventHandler(UNIQUE_HANDLER_ID);
/// ```
///
class EMContactEventHandler {
  ///
  ///
  ///
  /// 添加好友回调。
  ///
  final void Function(
    String userId,
  )? onContactAdded;

  ///
  ///
  ///
  /// 删除好友回调。
  ///
  final void Function(
    String userId,
  )? onContactDeleted;

  ///
  ///
  ///
  /// 好友申请回调。
  ///
  final void Function(
    String userId,
    String? reason,
  )? onContactInvited;

  ///
  ///
  ///
  /// 发出的好友申请被对方同意。
  ///
  final void Function(
    String userId,
  )? onFriendRequestAccepted;

  ///
  ///
  ///
  /// 发出的好友申请被对方拒绝。
  ///
  final void Function(
    String userId,
  )? onFriendRequestDeclined;

  ///
  ///
  ///
  /// 联系人事件监听。
  ///
  /// Param [onContactAdded] 添加好友回调。
  ///
  /// Param [onContactDeleted] 删除好友回调。
  ///
  /// Param [onContactInvited] 好友申请回调。
  ///
  /// Param [onFriendRequestAccepted] 发出的好友申请被对方同意。
  ///
  /// Param [onFriendRequestDeclined] 发出的好友申请被对方拒绝。
  ///
  EMContactEventHandler({
    this.onContactAdded,
    this.onContactDeleted,
    this.onContactInvited,
    this.onFriendRequestAccepted,
    this.onFriendRequestDeclined,
  });
}

///
///
///
/// 群组事件监听
///
/// 添加监听:
/// ```dart
///   EMClient.getInstance.groupManager.addEventHandler(UNIQUE_HANDLER_ID, EMGroupEventHandler());
/// ```
///
/// 移除监听:
/// ```dart
///   EMClient.getInstance.groupManager.removeEventHandler(UNIQUE_HANDLER_ID);
/// ```
///
class EMGroupEventHandler {
  ///
  ///
  ///
  /// 成员设置为管理员的回调。
  ///
  final void Function(
    String groupId,
    String admin,
  )? onAdminAddedFromGroup;

  ///
  ///
  ///
  /// 取消成员的管理员权限的回调。
  ///
  final void Function(
    String groupId,
    String admin,
  )? onAdminRemovedFromGroup;

  ///
  ///
  ///
  /// 全员禁言状态变化回调。
  ///
  final void Function(
    String groupId,
    bool isAllMuted,
  )? onAllGroupMemberMuteStateChanged;

  ///
  ///
  ///
  /// 成员加入群组白名单回调。
  ///
  final void Function(
    String groupId,
    List<String> members,
  )? onAllowListAddedFromGroup;

  ///
  ///
  ///
  /// 成员移出群组白名单回调。
  ///
  final void Function(
    String groupId,
    List<String> members,
  )? onAllowListRemovedFromGroup;

  ///
  ///
  ///
  /// 群公告更新回调。
  ///
  final void Function(
    String groupId,
    String announcement,
  )? onAnnouncementChangedFromGroup;

  ///
  ///
  ///
  /// 当前用户自动同意入群邀请的回调。
  /// 设置请见 [EMOptions.autoAcceptGroupInvitation].
  ///
  final void Function(
    String groupId,
    String inviter,
    String? inviteMessage,
  )? onAutoAcceptInvitationFromGroup;

  ///
  ///
  ///
  /// 当前用户收到群组被解散的回调。
  ///
  final void Function(
    String groupId,
    String? groupName,
  )? onGroupDestroyed;

  ///
  ///
  ///
  /// 当前用户收到对端用户同意入群邀请触发的回调。
  ///
  final void Function(
    String groupId,
    String invitee,
    String? reason,
  )? onInvitationAcceptedFromGroup;

  ///
  ///
  ///
  /// 当前用户收到群组邀请被拒绝的回调。
  /// 该回调是由当前用户收到对端用户拒绝入群邀请触发的。例如，用户 B 拒绝了用户 A 的群组邀请，用户 A 会收到该回调。
  ///
  final void Function(
    String groupId,
    String invitee,
    String? reason,
  )? onInvitationDeclinedFromGroup;

  ///
  ///
  ///
  /// 当前用户收到入群邀请的回调。
  ///
  final void Function(
    String groupId,
    String? groupName,
    String inviter,
    String? reason,
  )? onInvitationReceivedFromGroup;

  ///
  ///
  ///
  /// 群组成员主动退出回调。
  ///
  final void Function(
    String groupId,
    String member,
  )? onMemberExitedFromGroup;

  ///
  ///
  ///
  /// 新成员加入群组的回调。
  ///
  final void Function(
    String groupId,
    String member,
  )? onMemberJoinedFromGroup;

  ///
  ///
  ///
  /// 有成员被禁言回调。
  /// 用户禁言后，将无法在群中发送消息，但可查看群组中的消息，而黑名单中的用户无法查看和发送群组消息。
  ///
  final void Function(
    String groupId,
    List<String> mutes,
    int? muteExpire,
  )? onMuteListAddedFromGroup;

  ///
  ///
  ///
  /// 有成员被解除禁言的回调。
  ///
  final void Function(
    String groupId,
    List<String> mutes,
  )? onMuteListRemovedFromGroup;

  ///
  ///
  ///
  /// 转移群主权限的回调。
  ///
  final void Function(
    String groupId,
    String newOwner,
    String oldOwner,
  )? onOwnerChangedFromGroup;

  ///
  ///
  ///
  /// 对端用户接受当前用户发送的群组申请的回调。
  ///
  final void Function(
    String groupId,
    String? groupName,
    String accepter,
  )? onRequestToJoinAcceptedFromGroup;

  ///
  ///
  ///
  /// 对端用户拒绝群组申请的回调。
  ///
  final void Function(
    String groupId,
    String? groupName,
    String decliner,
    String? reason,
  )? onRequestToJoinDeclinedFromGroup;

  ///
  ///
  ///
  /// 对端用户接收群组申请的回调。
  ///
  final void Function(
    String groupId,
    String? groupName,
    String applicant,
    String? reason,
  )? onRequestToJoinReceivedFromGroup;

  ///
  ///
  ///
  /// 群组添加共享文件回调。
  ///
  final void Function(
    String groupId,
    EMGroupSharedFile sharedFile,
  )? onSharedFileAddedFromGroup;

  ///
  ///
  ///
  /// 群详情变更回调。
  ///
  final void Function(
    EMGroup group,
  )? onSpecificationDidUpdate;

  ///
  ///
  ///
  /// 群是禁用状态变更。
  ///
  final void Function(
    String groupId,
    bool isDisable,
  )? onDisableChanged;

  ///
  ///
  ///
  /// 群组删除共享文件回调。
  ///
  final void Function(
    String groupId,
    String fileId,
  )? onSharedFileDeletedFromGroup;

  ///
  ///
  ///
  /// 当前用户被移出群组时的回调。
  ///
  final void Function(
    String groupId,
    String? groupName,
  )? onUserRemovedFromGroup;

  ///
  ///
  ///
  /// 群组成员自定义属性有变更。
  ///
  /// Param [groupId] 群组 ID。
  ///
  /// Param [userId] 自定义属性变更的群成员的用户 ID。
  ///
  /// Param [attributes] 修改后的自定义属性，key-value 格式。
  ///
  /// Param [operatorId] 操作者的用户 ID。
  ///
  final void Function(
    String groupId,
    String userId,
    Map<String, String>? attributes,
    String? operatorId,
  )? onAttributesChangedOfGroupMember;

  ///
  ///
  ///
  /// 群组时间监听
  ///
  /// Param [onAdminAddedFromGroup] 成员设置为管理员的回调。
  ///
  /// Param [onAdminRemovedFromGroup] 取消成员的管理员权限的回调。
  ///
  /// Param [onAllGroupMemberMuteStateChanged] 全员禁言状态变化回调。
  ///
  /// Param [onAllowListAddedFromGroup] 成员加入群组白名单回调。
  ///
  /// Param [onAllowListRemovedFromGroup] 成员移出群组白名单回调。
  ///
  /// Param [onAnnouncementChangedFromGroup] 群公告更新回调。
  ///
  /// Param [onAutoAcceptInvitationFromGroup] 当前用户自动同意入群邀请的回调, 设置请见 [EMOptions.autoAcceptGroupInvitation]。
  ///
  /// Param [onGroupDestroyed] 当前用户收到群组被解散的回调。
  ///
  /// Param [onInvitationAcceptedFromGroup] 当前用户收到对端用户同意入群邀请触发的回调。
  ///
  /// Param [onInvitationDeclinedFromGroup] 当前用户收到群组邀请被拒绝的回调。
  ///
  /// Param [onInvitationReceivedFromGroup] 当前用户收到入群邀请的回调。
  ///
  /// Param [onMemberExitedFromGroup] 群组成员主动退出回调。
  ///
  /// Param [onMemberJoinedFromGroup] 新成员加入群组的回调。
  ///
  /// Param [onMuteListAddedFromGroup] 有成员被禁言回调, 用户禁言后，将无法在群中发送消息，但可查看群组中的消息，而黑名单中的用户无法查看和发送群组消息。
  ///
  /// Param [onMuteListRemovedFromGroup] 有成员被解除禁言的回调。
  ///
  /// Param [onOwnerChangedFromGroup] 转移群主权限的回调。
  ///
  /// Param [onRequestToJoinAcceptedFromGroup] 对端用户接受当前用户发送的群组申请的回调。
  ///
  /// Param [onRequestToJoinDeclinedFromGroup] 对端用户拒绝群组申请的回调。
  ///
  /// Param [onRequestToJoinReceivedFromGroup] 对端用户接收群组申请的回调。
  ///
  /// Param [onSharedFileAddedFromGroup] 群组添加共享文件回调。
  ///
  /// Param [onSharedFileDeletedFromGroup] 群组删除共享文件回调。
  ///
  /// Param [onUserRemovedFromGroup] 当前用户被移出群组时的回调。
  ///
  /// Param [onSpecificationDidUpdate] 群详情变更回调。
  ///
  /// Param [onDisableChanged] 群是禁用状态变更。
  ///
  /// Param [onAttributesChangedOfGroupMember] 群组成员自定义属性有变更。
  ///
  EMGroupEventHandler({
    this.onAdminAddedFromGroup,
    this.onAdminRemovedFromGroup,
    this.onAllGroupMemberMuteStateChanged,
    this.onAllowListAddedFromGroup,
    this.onAllowListRemovedFromGroup,
    this.onAnnouncementChangedFromGroup,
    this.onAutoAcceptInvitationFromGroup,
    this.onGroupDestroyed,
    this.onInvitationAcceptedFromGroup,
    this.onInvitationDeclinedFromGroup,
    this.onInvitationReceivedFromGroup,
    this.onMemberExitedFromGroup,
    this.onMemberJoinedFromGroup,
    this.onMuteListAddedFromGroup,
    this.onMuteListRemovedFromGroup,
    this.onOwnerChangedFromGroup,
    this.onRequestToJoinAcceptedFromGroup,
    this.onRequestToJoinDeclinedFromGroup,
    this.onRequestToJoinReceivedFromGroup,
    this.onSharedFileAddedFromGroup,
    this.onSharedFileDeletedFromGroup,
    this.onUserRemovedFromGroup,
    this.onSpecificationDidUpdate,
    this.onDisableChanged,
    this.onAttributesChangedOfGroupMember,
  });
}

///
///
///
/// 订阅用户状态变更监听
///
/// 添加监听:
/// ```dart
///   EMClient.getInstance.presenceManager.addEventHandler(UNIQUE_HANDLER_ID, EMPresenceEventHandler());
/// ```
///
/// 移除监听:
/// ```dart
///   EMClient.getInstance.presenceManager.removeEventHandler(UNIQUE_HANDLER_ID);
/// ```
///
class EMPresenceEventHandler {
  ///
  ///
  ///
  /// 收到被订阅用户的在线状态发生变化。
  ///
  final Function(List<EMPresence> list)? onPresenceStatusChanged;

  ///
  ///
  ///
  /// 订阅用户状态变更监听。
  ///
  EMPresenceEventHandler({
    this.onPresenceStatusChanged,
  });
}
